<?xml version="1.0"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="gwenview" version="68">

<MenuBar>
    <Menu name="file" >
        <Action name="file_save"/>
        <Action name="file_save_as"/>
        <Separator/>
        <Action name="reload"/>
        <Action name="file_print"/>
        <Action name="file_print_preview"/>
        <Separator/>
        <ActionList name="file_action_list"/>
        <Separator/>
        <Action name="add_folder_to_places"/>
        <Separator/>
        <Action name="quit"/>
    </Menu>
    <Menu name="view" >
        <Action name="browse" />
        <Action name="view" />
        <Separator/>
        <Action name="fullscreen" />
        <Separator/>
        <Action name="toggle_slideshow" />
        <Separator/>
        <Action name="view_actual_size"/>
        <Action name="view_zoom_to_fit"/>
        <Action name="view_zoom_in"/>
        <Action name="view_zoom_out"/>
        <Action name="view_toggle_birdeyeview"/>
        <Separator/>
        <Action name="view_background_colormode_auto"/>
        <Action name="view_background_colormode_light"/>
        <Action name="view_background_colormode_neutral"/>
        <Action name="view_background_colormode_dark"/>
        <Separator/>
        <Action name="sort_by"/>
        <Action name="thumbnail_details"/>
        <Separator/>
        <Action name="toggle_sidebar" />
    </Menu>
    <Menu name="edit" >
        <Action name="edit_undo" />
        <Action name="edit_redo" />
        <Separator/>
        <Action name="edit_cut"/>
        <Action name="edit_copy"/>
        <Action name="edit_paste"/>
        <Separator/>
        <Action name="edit_tags" />
        <Menu name="rating">
            <text>&amp;Rating</text>
            <Action name="rate_0" />
            <Action name="rate_1" />
            <Action name="rate_2" />
            <Action name="rate_3" />
            <Action name="rate_4" />
            <Action name="rate_5" />
        </Menu>
        <Separator/>
        <Action name="rotate_left" />
        <Action name="rotate_right" />
        <Action name="mirror" />
        <Action name="flip" />
        <Separator/>
        <Action name="resize" />
        <Action name="crop" />
        <Action name="brightness_contrast_gamma" />
        <Action name="red_eye_reduction" />
        <Action name="annotate" />
    </Menu>
    <Menu name="go" >
        <Action name="go_first"/>
        <Action name="go_previous"/>
        <Action name="go_next"/>
        <Action name="go_last"/>
        <Separator/>
        <Action name="go_up"/>
        <Separator/>
        <Action name="go_start_page"/>
    </Menu>
</MenuBar>

<ToolBar name="mainToolBar" noMerge="1">
    <text>Main Toolbar</text>
    <Action name="go_start_page" />
    <Separator/>
    <Action name="browse" />
    <Action name="view" />
    <Action name="align_with_sidebar" />
    <Separator/>
    <Action name="go_previous" />
    <Action name="go_next" />
    <Action name="toggle_operations_sidebar" />
    <Action name="share" />
    <Spacer/>
    <Action name="fullscreen" />
    <Action name="hamburger_menu"/>
</ToolBar>
<ActionProperties scheme="Default">
    <Action priority="0" name="fullscreen"/>
</ActionProperties>
</gui>
